# -*- perl -*-

# Copyright (c) 2010 AdCopy
# Author: Jeff Weisberg
# Created: 2010-May-13 17:06 (EDT)
# Function: documentation
#
# $Id: DC.pm,v 1.3 2011/01/12 19:29:21 jaw Exp $

package AC::DC;
use strict;
our $VERSION = 1.0;

=head1 NAME

AC::DC - Asynchronous IO Framework. plus.

=head1 SYNOPSIS

    use AC::DC::IO;

    ...

=head1 USAGE

Copy + Paste from the example code into your own code.

=head1 LICENSE

This software may be copied and distributed under the terms
found in the Perl "Artistic License".

A copy of the "Artistic License" may be found in the standard
Perl distribution.

=head1 BUGS

Too many to list here.

=head1 AUTHOR

    Jeff Weisberg - http://www.solvemedia.com/

=cut

1;
